/*===========================================================================*/
/* Module : low_level_initialization.c                                       */
/* Version: V1.00                                                            */
/*===========================================================================*/
/*                                  COPYRIGHT                                */
/*===========================================================================*/
/* Copyright (c) 2011 by Renesas Electronics Europe GmbH,                    */
/*               a company of the Renesas Electronics Corporation            */
/*===========================================================================*/
/* Purpose: basic microcontroller initialization                             */
/*                                                                           */
/*===========================================================================*/
/* In case of any question please do not hesitate to contact:                */
/*                                                                           */
/*        Technical Product Support                                          */
/*                                                                           */
/*        Renesas Electronics Europe GmbH                                    */
/*        Arcadiastrasse 10                                                  */
/*        D-40472 Duesseldorf, Germany                                       */
/*                                                                           */
/*        e-mail: device_support.rl78-eu@lm.renesas.com                      */
/*        FAX:   +49 - (0)211 / 65 03 - 12 79                                */
/*                                                                           */
/*===========================================================================*/
/* Warranty Disclaimer                                                       */
/*                                                                           */
/* Because the Product(s) is licensed free of charge, there is no warranty   */
/* of any kind whatsoever and expressly disclaimed and excluded by Renesas,  */
/* either expressed or implied, including but not limited to those for       */
/* non-infringement of intellectual property, merchantability and/or         */
/* fitness for the particular purpose.                                       */
/* NEC shall not have any obligation to maintain, service or provide bug     */
/* fixes for the supplied Product(s) and/or the Application.                 */
/*                                                                           */
/* Each User is solely responsible for determining the appropriateness of    */
/* using the Product(s) and assumes all risks associated with its exercise   */
/* of rights under this Agreement, including, but not limited to the risks   */
/* and costs of program errors, compliance with applicable laws, damage to   */
/* or loss of data, programs or equipment, and unavailability or             */
/* interruption of operations.                                               */
/*                                                                           */
/* Limitation of Liability                                                   */
/*                                                                           */
/* In no event shall Renesas be liable to the User for any incidental,       */
/* consequential, indirect, or punitive damage (including but not limited    */
/* to lost profits) regardless of whether such liability is based on breach  */
/* of contract, tort, strict liability, breach of warranties, failure of     */
/* essential purpose or otherwise and even if advised of the possibility of  */
/* such damages. Renesas shall not be liable for any services or products    */
/* provided by third party vendors, developers or consultants identified or  */
/* referred to the User by Renesas in connection with the Product(s) and/or  */
/* the Application.                                                          */
/*                                                                           */
/*===========================================================================*/
/* Environment:                                                              */
/*              Device:          R5F100LE                                    */
/*              Target Hardware: QB-R5F100LE-TB                              */
/*              IDE:             IAR Systems                                 */
/*                               Embedded Workbench for RL78 V1.xx           */
/*                                                                           */
/*===========================================================================*/
/* History:                                                                  */
/*              V1.00: Initial version                                       */
/*                                                                           */
/*===========================================================================*/


/*===========================================================================*/
/* Includes                                                                  */
/*===========================================================================*/
/* IAR Systems Includes                                                      */
#include <intrinsics.h>
#include <ior5f100LE.h>
#include <ior5f100LE_ext.h>


/* Application Includes                                                      */
#include <types.h>
#include <includes.h>
#include <macros.h>
#include <low_level_initialization.h>
#include <CG_timer.h>
/*===========================================================================*/
/* Constants                                                                 */
/*===========================================================================*/

/* Security Byte Definition                                                  */
__root __far const UCHAR SecuIDCode[SECU_ID_SIZE]   @ 0x00C4u = {
  0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu
};


/*===========================================================================*/
/* Prototypes                                                                */
/*===========================================================================*/
void WDT_Restart(void);
#pragma diag_suppress=Pm011
int __low_level_init (void);
#pragma diag_default=Pm011

/*===========================================================================*/
/* Functions                                                                 */
/*===========================================================================*/
#pragma diag_suppress=Pm011
int __low_level_init (void)
#pragma diag_default=Pm011
{

  /* -------------------------------------------------------------------------*/
  /* Peripheral I/O Redirection Configuration                                 */
  /* -------------------------------------------------------------------------*/
  PIOR = 0x00u;                     /* no redirection                         */

  /* -------------------------------------------------------------------------*/
  /* oscillator settings                                                      */
  /* -------------------------------------------------------------------------*/
  CMC     = 0x00u;   /* x1,x2,xt1 and xt2 as I/O pins,  low power oscillator  */
  MSTOP   = 1u;
  /*set internal high-speed oscillation clock fIH as main system clock fmain  */
  MCM0    = 0u;
  
  /* internal low-speed oscillation clock for real-time counter/interval timer*/
  OSMC    = 0x10u;

  /* set main system clock fmain as CPU/peripheral hardware clock             */
  CSS     = 0U;
  HIOSTOP = 0u;                   /* internal high-speed oscillator operating */
  XTSTOP  = 1u;                   /* XT1 oscillator stopped                   */

  /* -------------------------------------------------------------------------*/
  /* I/O Port Configuration                                                   */
  /* -------------------------------------------------------------------------*/

  PM7_bit.no0 = OUTPUT;               /* backlight                            */  
  PM7_bit.no1 = OUTPUT;               /* use port P7.1 as output              */
  PM7_bit.no2 = OUTPUT;               /* use port P7.2 as output              */
  PM7_bit.no3 = OUTPUT;               /* use port P7.3 as output              */  
  PM7_bit.no4 = OUTPUT;               /* use port P7.4 as output              */ 

  PM5_bit.no1 = INPUT;                /* use port P5.1 as input switch 3      */
  
  PM5_bit.no2 = OUTPUT;               /* use port P5.2 as output              */
  PM5_bit.no3 = OUTPUT;               /* use port P5.3 as output              */
  PM5_bit.no4 = OUTPUT;               /* use port P5.4 as output              */  
  PM5_bit.no5 = OUTPUT;               /* use port P5.5 as output              */ 
    
  PM6_bit.no2 = OUTPUT;               /* use port P6.2 as output              */
  PM6_bit.no3 = OUTPUT;               /* use port P6.3 as output              */
  
  P7_bit.no0 = _FALSE;               /* turn backlight off                   */
  P7_bit.no1 = _FALSE;               /* clear port register P7.1             */
  P7_bit.no2 = _FALSE;               /* clear port register P7.2             */
  P7_bit.no3 = _FALSE;               /* clear port register P7.3             */
  P7_bit.no4 = _FALSE;               /* clear port register P7.4             */ 

  P5_bit.no2 = _TRUE;              /* clear port register P5.2             */
  P5_bit.no3 = _TRUE;               /* clear port register P5.3             */
  P5_bit.no4 = _TRUE;               /* clear port register P5.4             */
  P5_bit.no5 = _TRUE;               /* clear port register P5.5             */ 
 
  P6_bit.no2 = _TRUE;               /* clear port register P6.2             */
  P6_bit.no3 = _TRUE;               /* clear port register P6.3             */

  
  PM14        = 0xFCu;                /* P147,P146 fixed level on TB -> input */

  /* -------------------------------------------------------------------------*/
  /* Timer Configuration                                                      */
  /* -------------------------------------------------------------------------*/

    
  TAU0EN  = 1u;                        /* supply input clock                  */
  TT0     = 0x0AFFu;                   /* stop all channels                   */
  TPS0    = 0x0008u;                   /* clock frequency                     */
  TMMK00  = 1u;                        /* disable INTTM00 interrupt           */
  TMIF00  = 0u;                        /* clear INTTM00 interrupt flag        */
  TMPR100 = 0u;                        /* Set INTTM00 high priority           */
  TMPR000 = 0u;

  /* Channel 0 used as interval timer */
  TMR00   =  0x0000u;
  TDR00   =  0x9C3Fu;
  TO0    &= ~0x0001u;
  TOE0   &= ~0x0001u;

  /* Mask channel 5 interrupt */
  TMMK05 = 1U;    /* disable INTTM05 interrupt */
  TMIF05 = 0U;    /* clear INTTM05 interrupt flag */
  
  /* Channel 5 used as PWM output function */
  
  /* Set TO05 pin */
  PM0_bit.no5 = OUTPUT;              /* use port P0.5 is DC motor           */
  P0_bit.no5 = _FALSE;               /* clear port register P7.1             */

  /* Channel 4 is used as master channel for PWM output function */    
  TMR04 = _0000_TAU_CLOCK_SELECT_CKM0 | _0000_TAU_CLOCK_MODE_CKS | _0800_TAU_COMBINATION_MASTER | _0000_TAU_TRIGGER_SOFTWARE | _0001_TAU_MODE_PWM_MASTER;
  TDR04 = 0x0080; 
  TO0 &= ~_0010_TAU_CH4_OUTPUT_VALUE_1;
  TOE0 &= ~_0010_TAU_CH4_OUTPUT_ENABLE;
    
  /* Channel 5 is used as slave channel for PWM output function */
  TMR05 = _0000_TAU_CLOCK_SELECT_CKM0 | _0000_TAU_CLOCK_MODE_CKS | _0000_TAU_COMBINATION_SLAVE | _0400_TAU_TRIGGER_MASTER_INT | _0009_TAU_MODE_PWM_SLAVE;
  TDR05 = 0x0040; 
  TOM0 |= _0020_TAU_CH5_OUTPUT_COMBIN;
  TOL0 &= ~_0020_TAU_CH5_OUTPUT_LEVEL_L;
  TO0 &= ~_0020_TAU_CH5_OUTPUT_VALUE_1;
        
  TMIF04 = 0U;    /* clear INTTM04 interrupt flag */
  TMMK04 = 0U;    /* enable INTTM04 interrupt */
    
  TMIF05 = 0U;    /* clear INTTM05 interrupt flag */
  TMMK05 = 0U;    /* enable INTTM05 interrupt */    
  
  TMIF00  = 0u;                        /* clear INTTM00 interrupt flag        */
  TMMK00 &= 0u;                        /* enable INTTM00 interrupt            */
  TS0    |= 0x0001u;                   /* start timer TM00                    */

  /* -------------------------------------------------------------------------*/
  /* Watchdog Timer Configuration                                             */
  /* -------------------------------------------------------------------------*/
  WDTIMK  = 1u;                        /* disable INTWDTI interrupt           */
  WDTIIF  = 0u;                        /* clear INTWDTI interrupt flag        */
  WDTIPR1 = 0u;                        /* Set INTWDTI high priority           */
  WDTIPR0 = 0u;
  WDTIMK  = 0u;                        /* enable INTWDTI interrupt            */

  /* -------------------------------------------------------------------------*/
  /* External Interrupt Configuration                                         */
  /* -------------------------------------------------------------------------*/
  PPR10         = 0u;                 /* Set INTP0 level 1 priority           */
  PPR00         = 1u;
  EGN0_bit.no0  = _TRUE;               /* falling edge detection               */
  PIF0          = _FALSE;              /* clear interrupt request flag         */
  PMK0          = IRQ_ENABLE;         /* enable ext. INTP0 interrupt          */
  
  /* Set INTP2 low priority */
  PPR12 = 1U;
  PPR02 = 1U;
  
  EGN0_bit.no2 = _TRUE;               /* falling edge detection               */
  PM5 |= 0x02U;
  PIF2 = _FALSE;                      /* clear interrupt request flag         */
  // PMK2 = IRQ_ENABLE;	              /* enable INTP2 interrupt */
  
  /* Set RxD2 pin */
  PM1 |= 0x10U;
  
  /* Set TxD2 pin */
  P1 |= 0x08U;
  PM1 &= 0xF7U;

  /* -------------------------------------------------------------------------*/
  /* Safety Configuration                                                     */
  /* -------------------------------------------------------------------------*/
  CRC0CTL = 0x00U;           /* disable CRC calculation                       */
  IAWCTL  = 0x80U;           /* enable detection of invalid memory access     */

  return (int)(ENABLE_VAR_INIT);
}


void WDT_Restart(void) {
  WDTE = 0xACU;                        /* restart watchdog timer              */
}
