// peripherals.h
#ifndef _PERIPHERASL_H
#define _PERIPHERALS_H

/* Accelerometer constants */
#define ACCEL_ADDR                          0x3A
#define ACCEL_REG_PWRCTL                    0x2D
#define ACCEL_REG_DATASTART                 0x32
#define ACCEL_CTL_MEASURE                   0x08

/* Temp sensor constants */
#define TEMP_ADDR	                    0x90
#define TEMP_REG	                    0x00

/* Light sensor */
#define LS_ADDR                             0x72
#define LS_INSTR                            0x51
#define LUX_NIGHT_VALUE                     0x10                                /* not much sun/light anymore at this value */

/* eeprom */
#define EEPROM_ADDR                         0xA0u   // Slave Address (Write)

/* compass */
#define COMPASS_ADDR_7BIT                   0x1E
#define COMPASS_ADDR_8BIT_READ              0x3D
#define COMPASS_ADDR_8BIT_WRITE             0x3C
#define COMPASS_CONFIG_REGISTER_A           0
#define COMPASS_CONFIG_REGISTER_B           1
#define COMPASS_MODE_REGISTER               2
#define COMPASS_DATA_OUTPUT_X_MSB           3
#define COMPASS_DATA_OUTPUT_X_LSB           4
#define COMPASS_DATA_OUTPUT_Z_MSB           5
#define COMPASS_DATA_OUTPUT_Z_LSB           6
#define COMPASS_DATA_OUTPUT_Y_MSB           7
#define COMPASS_DATA_OUTPUT_Y_LSB           8
#define COMPASS_STATUS_REGISTER             9
#define COMPASS_IDENTIFICATION_REGISTER_A   10
#define COMPASS_IDENTIFICATION_REGISTER_B   11
#define COMPASS_IDENTIFICATION_REGISTER_C   12


#define COMPASS_STATUS_RDY                  1


void readGPS(float *idealAzimuth, float *idealElevation);
void receiveNMEAMessage();
void readCompass(short *yaw, signed int *compass_x, signed int *compass_y);
void readTemp(float *temp);
void readLightSensor(unsigned short *temp);
void readAccelerometer(signed int *roll, signed int *pitch);
void initPeripherals();

typedef struct
{
 float          latitude;         /* Latitude in degrees */
 float          longitude;        /* Longitude in degrees */
 unsigned char  sats;             /* number of satelites */
 int            quality;          /* quality of GPS signal */
 double         hdop;             /* Horizontal Dilution of Precision */
 double         geoid;            /* Antenna altitude above/below mean sea level (geoid) */
}GpsLogData;

extern GpsLogData gpsLogData;

#endif //_PERIPHERALs_H