/*
*********************************************************************************************************
*
*                                           Compass calibration
*                                     
*                                           Renesas RL78
*                                               on the
*                                     YRDKRL78G13 Evaluation Board
*
* Filename      : calibration.c
* Version       : V1.00
*********************************************************************************************************
*/

/*
*******************************************************************************
** Include files
*******************************************************************************
*/
#include <includes.h>
#include <string.h>
#include <calibration.h>
#include <peripherals.h>
#include <monitor.h>
#include <controller.h>
#include <calibration.h>
#include <CG_macrodriver.h> 
#include <CG_userdefine.h>
#include <CG_serial.h>

/*
*******************************************************************************
**  Local define
*******************************************************************************
*/
CalibrationData calibrationData = {0,0,0,0};           

/*
*********************************************************************************************************
*                                          calibrateCompass
*
* Description : Turns the platform 360 degrees, calculates the X and Y offset and stores it in EEPROM
*
* Arguments   : none
*
* Returns     : none
*
* Notes       : should be called from a task that's able to send (log) messages !
*********************************************************************************************************
*/
void calibrateCompass()
{
  int steps = 0;
  int initialHeading = 0;
  signed long x=0,y=0;
  OS_ERR err;
  short yaw;
  signed int compass_x;
  signed int compass_y;
  
  for (int i=0;i < 5;i++)
  {
    readCompass(&yaw, &compass_x, &compass_y);                             /* read current position */
    initialHeading += (int) yaw;                     
  }
  initialHeading /= 5;
  
  while (steps < 85 || abs(initialHeading - yaw ) > 5)                    /* check if we're rotating and haven't done a full circle. Full circle is about 100 steps */
  {  
     x += compass_x;
     y += compass_y;
     SendMessage(CMD_ROTATE,0);                                           /* Send rotate cmd to controller */
     sprintf((char *)timeStamp,"COMP. CALIBRATION");                      /* tell log it's a calibration message */
     sprintf((char *)logBuffer,"|CC|%i|%i|%i|\n",yaw,compass_x,compass_y);   /* output values to log */
     SendMessage(CMD_STORE_LOG_MESSAGE,logBuffer);

     OSTimeDlyHMSM(0u, 0u, 2u, 0u,                                        /* let everything setle down */
                     OS_OPT_TIME_HMSM_STRICT, 
                      &err); 
     readCompass(&yaw, &compass_x, &compass_y);                           /* perform new compass measurement */
     steps++;       
  }
  
  calibrationData.calibrationX = (long) x / steps;                                 /* average x measurements and store compensation */
  calibrationData.calibrationY = (long) y / steps;                                 /* average y measurements and store compensation */
  sprintf((char *)timeStamp,"CALIBRATION DONE");                         
  sprintf((char *)logBuffer,"XC|%i|YC|%i\n",calibrationData.calibrationX,calibrationData.calibrationY);   /* log calibration results */
  SendMessage(CMD_STORE_LOG_MESSAGE,logBuffer);
  WriteIICBuf(EEPROM_ADDR, (BYTE *) &calibrationData, sizeof(calibrationData));                      /* Write calibration data to EEPROM */
}



/*
*********************************************************************************************************
*                                          readCalibrationData
*
* Description : Reads calibration data from EEPROM
*
* Arguments   : none
*
* Returns     : CalibrationData struct that holds X and Y compensation
*
* Notes       : 
*********************************************************************************************************
*/
CalibrationData *readCalibrationData()
{      
        WriteIICBuf(EEPROM_ADDR, (BYTE *) &calibrationData, 2);                           /* write address */
        ReadIICBuf(EEPROM_ADDR, ((BYTE *) &calibrationData)+2,  sizeof(calibrationData)); /* read contents */
        return &calibrationData;
}